function [w4_x, w4_y, w4_z] = computeVelocity(initialVx, initialVy, initialVz, force_x, force_y, force_z, sizeOfcells, visc, dt, t)

%dt = constant
%sizeOfcells = [size_x size_y size_z]

%defining current velocity vector
w0_x = initialVx;
w0_y = initialVy;
w0_z = initialVz;

%finding size of velocity vector
sizew0 = [ size(initialVx,1) size(initialVx,2) size(initialVx,3)];

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% 1. advect - transport
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 

%defining new velociy vector
w1_x = zeros(sizew0); %NEW velocity after ADVECT
w1_y = w1_x;
w1_z = w1_x;

%calculating w1_x,y,z from w0_x,y,z
for i = 1:sizew0(1)
    for j = 1:sizew0(2)
        for k = 1:sizew0(3)
%           simplest stright line particle trace
            temp_prev_point = traceBack([w0_x(i, j, k) w0_y(i, j, k) w0_z(i, j, k)], [i j k], dt, sizeOfcells, sizew0);
            w1_x(i, j, k) = w0_x(temp_prev_point(1), temp_prev_point(2), temp_prev_point(3));
            w1_y(i, j, k) = w0_y(temp_prev_point(1), temp_prev_point(2), temp_prev_point(3));
            w1_z(i, j, k) = w0_z(temp_prev_point(1), temp_prev_point(2), temp_prev_point(3));
        end
    end
end

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% 2. add force
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 

w2_x = w1_x + (dt*force_x);
w2_y = w1_y + (dt*force_y);
w2_z = w1_z + (dt*force_z);

% w2_x(1,:,:)
% w2_y(:,2,:)
% w2_z(:,:,1)
% 
% error('heyhey')
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% 3. FFT transform
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 

%w2_hat = w2 in fourier domain
w2_x_hat = fftshift(fftn(w2_x));
w2_y_hat = fftshift(fftn(w2_y));
w2_z_hat = fftshift(fftn(w2_z));

% w2_x_hat(1,:,:)
% error('heyhey')
% w2_y_hat(:,2,:)
% error('heyhey')
% w2_z_hat(:,:,1)
% error('heyhey')
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% 4. diffusion
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

w3_x_hat = zeros(sizew0);
w3_y_hat = zeros(sizew0);
w3_z_hat = zeros(sizew0);

for a = 1:sizew0(1)
    for b = 1:sizew0(2)
        for c = 1:sizew0(3)
            k_hat = [a-(sizew0(1)/2)-1,b-(sizew0(2)/2)-1,c-(sizew0(3)/2)-1];
            divisor = 1 + visc*dt*(norm(k_hat)^2);
            
            w3_x_hat(a,b,c) = w2_x_hat(a,b,c) / divisor;
            w3_y_hat(a,b,c) = w2_y_hat(a,b,c) / divisor;
            w3_z_hat(a,b,c) = w2_z_hat(a,b,c) / divisor;
        end
    end
end

% w3_x_hat(1,:,:)
% error('heyhey')
% w3_y_hat(:,2,:)
% error('heyhey')
% w3_z_hat(:,:,1)
% error('seseresrv')

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% 5. projection
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

w4_x_hat = zeros(sizew0);
w4_y_hat = zeros(sizew0);
w4_z_hat = zeros(sizew0);

for a = 1:sizew0(1)
    for b = 1:sizew0(2)
        for c = 1:sizew0(3)
            k_hat = [a-(sizew0(1)/2)-1,b-(sizew0(2)/2)-1,c-(sizew0(3)/2)-1];
            k_hat_squared = norm(k_hat)^2;
            if k_hat_squared == 0
                coefficient = 0;
            else
                coefficient = dot(k_hat, [w3_x_hat(a,b,c),w3_y_hat(a,b,c),w3_z_hat(a,b,c)]) / k_hat_squared;
            end
            
            w4_x_hat(a,b,c) = w3_x_hat(a,b,c) - k_hat(1)*coefficient;
            w4_y_hat(a,b,c) = w3_y_hat(a,b,c) - k_hat(2)*coefficient;
            w4_z_hat(a,b,c) = w3_z_hat(a,b,c) - k_hat(3)*coefficient;
        end
    end
end

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% 6. transform back
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

w4_x = real(ifftn(ifftshift(w4_x_hat)));
w4_y = real(ifftn(ifftshift(w4_y_hat)));
w4_z = real(ifftn(ifftshift(w4_z_hat)));

