clf;
clear;
clc;

noOfTimeSteps = 1000;
dt = 0.001;
visc = 10;
noOfParticles = 1000;
plus_particlePerTime = 5;

%Define space
lengthOfSpace = [10 10 10];

noOfcells = [10 10 10];

maxVelocity = 1;

sizeofcells = lengthOfSpace./noOfcells;

% s = RandStream('mt19937ar', 'Seed', 1);
% RandStream.setGlobalStream(s);

%Generate random initial velocity
prevVx = zeros(noOfcells(1), noOfcells(2), noOfcells(3));
prevVy = prevVx;
prevVz = prevVx;

for i = 1:noOfcells(1)
    for j = 1:noOfcells(2)
        for k = 1:noOfcells(3) %equal probability of being both positive and negative
            prevVx(i,j,k) = (2*rand - 1)*maxVelocity;
            prevVy(i,j,k) = (2*rand - 1)*maxVelocity;
            prevVz(i,j,k) = (2*rand - 1)*maxVelocity;
        end
    end
end

%Generate current velocity matrix
currentVx = zeros(noOfcells(1), noOfcells(2), noOfcells(3));
currentVy = currentVx;
currentVz = currentVx;

%Generate force
Forcefactor = 100;

force_x = zeros(noOfcells(1), noOfcells(2), noOfcells(3));
force_y = zeros(noOfcells(1), noOfcells(2), noOfcells(3));
force_z = zeros(noOfcells(1), noOfcells(2), noOfcells(3));

radians_towards_centre = 5*pi/180;

rotation_matrix = [cos(radians_towards_centre) -sin(radians_towards_centre);sin(radians_towards_centre) cos(radians_towards_centre)];

for i = 1:noOfcells(1)
    for j = 1:noOfcells(2)
        for k = 1:noOfcells(3)
            %circular antisclockwise force
            Rr = norm([i-5 j-5 0].*lengthOfSpace); %magnitude of force depends on r from centre
            if i-5 == 0
                theta = pi/2;
            else
                theta = atan2(lengthOfSpace(2)*(j-5),lengthOfSpace(1)*(i-5));
            end
            Fx = Forcefactor*Rr*-sin(theta); %rotate slightly towards centre
            Fy = Forcefactor*Rr*cos(theta);
            newF = rotation_matrix*[Fx; Fy];
            force_x(i,j,k) = newF(1);
            force_y(i,j,k) = newF(2);
            force_z = Forcefactor*Forcefactor/(Rr+1);
        end
    end
end

%Generate random particle initial position

% particle = zeros(noOfParticles, 3);
% particle = rand(noOfParticles+2000, 3);

% particle(1:noOfParticles,1) = particle(1:noOfParticles,1)*lengthOfSpace(1);
% particle(1:noOfParticles,2) = particle(1:noOfParticles,2)*lengthOfSpace(2);
% particle(1:noOfParticles,3) = particle(1:noOfParticles,3)*lengthOfSpace(3)/4;

particle = rand(noOfParticles+1000, 3); %particles start near centre of xy plane, with low z value

particle(1:noOfParticles,1) = particle(1:noOfParticles,1)+4;
particle(1:noOfParticles,2) = particle(1:noOfParticles,2)+4;

% error('hgfd')
t = 0;
numberofparticlesSoFar = 10;
frame = 0;

for time = t:noOfTimeSteps
    [currentVx, currentVy, currentVz] = computeVelocity(prevVx, prevVy, prevVz, force_x, force_y, force_z, sizeofcells, visc, dt, t);
        %save png
    if mod(time,2) == 0
       frame = frame+1;   
%         %quiver plot velocity field
%         clf;
%         quiverPlot3D(currentVx, currentVy, currentVz, sizeofcells);

        %scatter plot all particles
        clf;
        hold on
        subplot(1, 2, 1);
        scatter3(particle(1:numberofparticlesSoFar,1),particle(1:numberofparticlesSoFar,2),particle(1:numberofparticlesSoFar,3),4,'m')
        set(gca,'plotboxaspectratio',[2 2 2])
        xlabel('X axis')
        ylabel('Y axis')
        zlabel('Z axis')
        title(['Number Of Particles = ', num2str(numberofparticlesSoFar)])
        view([0 90])
        axis([0 11 0 11 0 11])
        grid on;
        grid minor;

        subplot(1, 2, 2);
        scatter3(particle(1:numberofparticlesSoFar,1),particle(1:numberofparticlesSoFar,2),particle(1:numberofparticlesSoFar,3),4,'m')
        set(gca,'plotboxaspectratio',[2 2 2])
        xlabel('X axis')
        ylabel('Y axis')
        zlabel('Z axis')
        title(['Number Of Particles = ', num2str(numberofparticlesSoFar)])
        view([360 90])
        axis([0 10 0 10 0 10])
        grid on;
        grid minor
% 
% %         pause(0.01)
        drawnow
%         saveas(gcf, ['/Users/Matt/Documents/UC Berkeley/Fall 2013/EPS 109/final project/stable_fluid/7_particle_4_high_visc/p4_highv_', sprintf('%03d',frame)], 'png')
        hold off
    end
    %add particles per time frame
    if numberofparticlesSoFar < size(particle,1)
        numberofparticlesSoFar = numberofparticlesSoFar +plus_particlePerTime;
    else
        numberofparticlesSoFar;
    end
    
    for i = 1:numberofparticlesSoFar
        particle(i,:) = computeNextPosition(currentVx, currentVy, currentVz, particle(i,:), noOfcells, lengthOfSpace, dt);
    end
    
    %reset current as previous before next time step
    prevVx = currentVx;
    prevVy = currentVy;
    prevVz = currentVz; 
end